<?php

/**
 * Generates a popular posts list from shortcode
 *
 * @access public
 * @param mixed $atts
 * @return void
 */
function nnr_related_posts_shortcode($atts) {

	$args = shortcode_atts( array(
		'id'							=> 'nnrobots_related_posts_0',
        'title'							=> 'Related Posts',
		'display_in'					=> 'post',
		'location'						=> 'bottom',
		'num_of_columns'                => '2',
		'related_by'					=> 'category',
		'default_thumbnail'             => '',
		'layout'                        => 'inline_card',
        'grid_size'                     => 'medium',
        'show_comments'					=> false,
        'show_nav'						=> true,
        'show_category'                 => false,
        'show_author'                   => false,
        'show_post_date'                => false,
        'color'                         => '#FFFFFF',
        'text_color'                    => '#000000',
        'fancy_colors'                  => '#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982',
        'num_of_posts'                  => '6',
        'add_recent'                  	=> false,
        'time_range'					=> 'all_time',
        'order'							=> 'ASC',
        'order_by'						=> 'date',
        'async'                         => true,
        'style_override'                => true,
    ), $atts );

	$settings = get_option(NNRobots_Related_Posts::$prefix . 'settings');

	// Default values

	if ($settings === false) {
		$settings = NNRobots_Related_Posts::$default;
	}

	global $post;

    // Do not display shortcode if the post is displayed within a loop

    if (!is_singular()) {
		return null;
    }

    if (isset($args['show_comments']) && $args['show_comments'] == 'true') {
	    $args['show_comments'] = true;
    } else {
	    $args['show_comments'] = false;
    }

    if (isset($args['show_category']) && $args['show_category'] == 'true') {
	    $args['show_category'] = true;
    } else {
	    $args['show_category'] = false;
    }

    if (isset($args['show_author']) && $args['show_author'] == 'true') {
	    $args['show_author'] = true;
    } else {
	    $args['show_author'] = false;
    }

    if (isset($args['show_post_date']) && $args['show_post_date'] == 'true') {
	    $args['show_post_date'] = true;
    } else {
	    $args['show_post_date'] = false;
    }

    // Format data

	list($r, $g, $b)                               = sscanf($args['color'], "#%02x%02x%02x");
	list($text_r, $text_g, $text_b)                = sscanf($args['text_color'], "#%02x%02x%02x");

	$content = '';

	if (isset($args['async']) && $args['async'] == 'true') {
		$content .= '<script type="text/javascript">
		jQuery(document).ready(function($) {

			var data = {
				"action": "nnr_display_related_posts",
				"instance" : ' . json_encode($args) . ',
				"post" : ' . strip_shortcodes(json_encode($post)) . ',
				"widget_id" : "' . $args['id'] . '",
				"shortcode": true,
			};

			$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
				var data = jQuery.parseJSON(response);
				$("#" + data.widget_id).append(data.content).hide().fadeIn();';

					if ($args['layout'] == 'masonry') {
						$content .= '$("#" + data.widget_id + " .' . NNRobots_Related_Posts::$prefix_dash . 'masonry").justifiedGallery({
							lastRow: "justify",
							rowHeight: 150,
						});';
					}

					if ($args['layout'] == 'slider') {
						$content .= '$("#" + data.widget_id + " .' . NNRobots_Related_Posts::$prefix_dash . 'slider").flexslider({
							animation: "fade",';

						if (isset($args[ 'show_nav' ]) && $args[ 'show_nav' ]) {
							$content .= 'directionNav: true,';
						} else {
							$content .= 'directionNav: false,';
						}

						$content .= '});';
					}

			$content .= '});
		});
		</script>
		<div id="' . $args['id'] . '" class="' . NNRobots_Related_Posts::$prefix_dash . 'shortcode"></div>';
	} else {
		$content = '<script type="text/javascript">
			jQuery(document).ready(function($) {';

				if ($args['layout'] == 'masonry') {
					$content .= '$("#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'masonry").justifiedGallery({
						lastRow: "justify",
						rowHeight: 150,
					});';
				}

				if ($args['layout'] == 'slider') {
					$content .= '$("#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'slider").flexslider({
						animation: "fade",';

					if (isset($args[ 'show_nav' ]) && $args[ 'show_nav' ]) {
						$content .= 'directionNav: true,';
					} else {
						$content .= 'directionNav: false,';
					}

					$content .= '});';
				}

		$content .= '});
		</script>';
		$content .= NNRobots_Related_Posts_Widget::display_related_posts($args, json_encode($post), true);
	}

	if ($args['style_override'] == 'true') {

		$content .= '<style>

			/* Background Color */

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'inline-card-wrap {
				background-color: rgb(' . $r. ', ' . $g. ', ' . $b. ');
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'stacked-card-wrap {
				background-color: rgb(' . $r. ', ' . $g. ', ' . $b. ');
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-tile-item .' . NNRobots_Related_Posts::$prefix_dash . 'title {
				background-color: rgba(' . $r. ', ' . $g. ', ' . $b. ', 0.9);
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'colored-tabs-item {
				background-color: rgb(' . $r. ', ' . $g. ', ' . $b. ');
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-item {
				background-color: rgb(' . $r. ', ' . $g. ', ' . $b. ');
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-image .' . NNRobots_Related_Posts::$prefix_dash . 'title,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-image .' . NNRobots_Related_Posts::$prefix_dash . 'meta-data {
				background-color: rgba(' . $r. ', ' . $g. ', ' . $b. ', 0.5);
			}

			/* Text Color */

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'title {font-weight: bold;}

			#' . $args['id'] . ' a,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'title,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'comment-count,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'meta-data {
				color: rgb(' . $text_r. ', ' . $text_g. ', ' . $text_b. ');
			}

		</style>';
	} else {
		$content .= '<style>

			/* Background Color */

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-tiles .' . NNRobots_Related_Posts::$prefix_dash . 'title {
				background-color: rgba(0, 0, 0, 0.9);
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-image .' . NNRobots_Related_Posts::$prefix_dash . 'title,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-image .' . NNRobots_Related_Posts::$prefix_dash . 'meta-data {
				background-color: rgba(0, 0, 0, 0.5);
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-tiles .' . NNRobots_Related_Posts::$prefix_dash . 'title {
				color: white;
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-tiles .' . NNRobots_Related_Posts::$prefix_dash . 'comment-count,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-tiles .' . NNRobots_Related_Posts::$prefix_dash . 'meta-data *  {
				color: white;
			}

			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-content a,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-content .' . NNRobots_Related_Posts::$prefix_dash . 'title
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-content .' . NNRobots_Related_Posts::$prefix_dash . 'comment-count,
			#' . $args['id'] . ' .' . NNRobots_Related_Posts::$prefix_dash . 'image-flip-content .' . NNRobots_Related_Posts::$prefix_dash . 'meta-data {
				color: white;
			}

		</style>';
	}

	// Include masonry files

	if ($args['layout'] == 'masonry') {
		wp_enqueue_script(NNRobots_Related_Posts::$prefix_dash . 'justified_gallery_js', plugins_url('core/includes/masonry/jquery.justifiedGallery.min.js', __FILE__), array('jquery'));
		wp_enqueue_style(NNRobots_Related_Posts::$prefix_dash . 'justified_gallery_css', plugins_url('core/includes/masonry/justifiedGallery.min.css', __FILE__));
	}

	// Include slider files

	if ($args['layout'] == 'slider') {
		wp_enqueue_script(NNRobots_Related_Posts::$prefix_dash . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
		wp_enqueue_style(NNRobots_Related_Posts::$prefix_dash . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
	}

	wp_enqueue_style(NNRobots_Related_Posts::$prefix_dash . 'widget_css', plugins_url('core/css/nnr-related-posts.css', __FILE__));
	wp_enqueue_style( 'dashicons' );

	return $content;
}