<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action( 'widgets_init', function(){
     register_widget( 'NNRobots_Related_Posts_Widget' );
});

/*
 * AJAX hooks
 */

add_action( 'wp_ajax_nnr_display_related_posts', 			array('NNRobots_Related_Posts_Widget', 'ajax_display_related_posts'));
add_action( 'wp_ajax_nopriv_nnr_display_related_posts', 	array('NNRobots_Related_Posts_Widget', 'ajax_display_related_posts'));
add_action( 'init', 										array('NNRobots_Related_Posts_Widget', 'load_textdomain'));

/**
 * Adds NNRobots_Related_Posts_Widget widget.
 */
class NNRobots_Related_Posts_Widget extends WP_Widget {

	/**
	 * text_domain
	 *
	 * (default value: 'nnrobots-related-posts')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $text_domain = 'nnrobots-related-posts';

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_related_posts_widget_')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $prefix = 'nnrobots_related_posts_widget_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-rlp-')
	 *
	 * @var string
	 * @access private
	 * @static
	 */
	private static $prefix_dash = 'nnr-rlp-';

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdomain() {
		load_plugin_textdomain(self::$text_domain, false, NNROBOTS_RELATED_POSTS_PLUGIN_DIR . '/languages');
	}

	/**
	 * Register widget with WordPress.
	 */
	function __construct() {
		parent::__construct(
			self::$prefix_dash . 'widget', // Base ID
			__('Related Posts Pro', self::$text_domain), // Name
			array( 'description' => __( 'Display related posts', self::$text_domain ), ) // Args
		);
	}

	/**
	 * Display all content using AJAX as to no effect page load times
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	public static function ajax_display_related_posts() {

		// Convert all strings to bool when needed

		if ( isset($_POST['instance']['show_comments']) && is_string($_POST['instance']['show_comments']) &&
			( $_POST['instance']['show_comments'] == 'true' || $_POST['instance']['show_comments'] == 'on' ) ) {
			$_POST['instance']['show_comments'] = true;
		} else {
			$_POST['instance']['show_comments'] = false;
		}

		if ( isset($_POST['instance']['show_nav']) && is_string($_POST['instance']['show_nav']) &&
			( $_POST['instance']['show_nav'] == 'true' || $_POST['instance']['show_nav'] == 'on' ) ) {
			$_POST['instance']['show_nav'] = true;
		} else {
			$_POST['instance']['show_nav'] = false;
		}

		if ( isset($_POST['instance']['show_category']) && is_string($_POST['instance']['show_category']) &&
			( $_POST['instance']['show_category'] == 'true' || $_POST['instance']['show_category'] == 'on' ) ) {
			$_POST['instance']['show_category'] = true;
		} else {
			$_POST['instance']['show_category'] = false;
		}

		if ( isset($_POST['instance']['show_author']) && is_string($_POST['instance']['show_author']) &&
			( $_POST['instance']['show_author'] == 'true' || $_POST['instance']['show_author'] == 'on' ) ) {
			$_POST['instance']['show_author'] = true;
		} else {
			$_POST['instance']['show_author'] = false;
		}

		if ( isset($_POST['instance']['show_post_date']) && is_string($_POST['instance']['show_post_date']) &&
			( $_POST['instance']['show_post_date'] == 'true' || $_POST['instance']['show_post_date'] == 'on' ) ) {
			$_POST['instance']['show_post_date'] = true;
		} else {
			$_POST['instance']['show_post_date'] = false;
		}

		if ( isset($_POST['instance']['add_recent']) && is_string($_POST['instance']['add_recent']) &&
			( $_POST['instance']['add_recent'] == 'true' || $_POST['instance']['add_recent'] == 'on' ) ) {
			$_POST['instance']['add_recent'] = true;
		} else {
			$_POST['instance']['add_recent'] = false;
		}

		if ( isset($_POST['instance']['async']) && is_string($_POST['instance']['async']) &&
			( $_POST['instance']['async'] == 'true' || $_POST['instance']['async'] == 'on' ) ) {
			$_POST['instance']['async'] = true;
		} else {
			$_POST['instance']['async'] = false;
		}

		if ( isset($_POST['instance']['style_override']) && is_string($_POST['instance']['style_override']) &&
			( $_POST['instance']['style_override'] == 'true' || $_POST['instance']['style_override'] == 'on' ) ) {
			$_POST['instance']['style_override'] = true;
		} else {
			$_POST['instance']['style_override'] = false;
		}

		if (isset($_POST['shortcode']) && $_POST['shortcode']) {
			echo json_encode(array('content' => self::display_related_posts($_POST['instance'], $_POST['post'], true), 'widget_id' => $_POST['widget_id']));
		} else {
			echo json_encode(array('content' => self::display_related_posts($_POST['instance'], $_POST['post']), 'widget_id' => $_POST['widget_id']));
		}

		die();
	}

	/**
	 * Front-end display of widget.
	 *
	 * @see WP_Widget::widget()
	 *
	 * @param array $args     Widget arguments.
	 * @param array $instance Saved values from database.
	 */
	public function widget( $args, $instance ) {

		if ( !is_singular() ) {
			return;
		}

		global $post;

		$title = apply_filters( 'widget_title', $instance['title'] );

		echo $args['before_widget'];

		if ( ! empty( $title ) ) {
			echo $args['before_title'] . $title . $args['after_title'];
		}

		if ( !isset($instance['color']) || $instance['color'] == '') {
			$instance['color'] = '#000000';
		}

		list($r, $g, $b)                               = sscanf($instance['color'], "#%02x%02x%02x");
		list($text_r, $text_g, $text_b)                = sscanf($instance['text_color'], "#%02x%02x%02x");

		if (isset($instance['async']) && $instance['async']) {
			echo '<script type="text/javascript">
			jQuery(document).ready(function($) {

				var data = {
					"action": "nnr_display_related_posts",
					"instance" : ' . json_encode($instance) . ',
					"widget_id" : "' . $this->id . '",
					"post": ' . json_encode($post) . '
				};

				$.post("' . admin_url( 'admin-ajax.php' ) . '", data, function(response) {
					response = jQuery.parseJSON(response);
					$("#" + response.widget_id).append(response.content).hide().fadeIn();';

					if ($instance['layout'] == 'masonry') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'masonry").justifiedGallery({
							lastRow: "justify",
							rowHeight: 75,
						});';
					}

					if ($instance['layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'slider").flexslider({
							animation: "fade",';

						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo '});
			});
			</script>';
		} else {
			echo self::display_related_posts($instance, $post);
			echo '<script type="text/javascript">
				jQuery(document).ready(function($) {';

					if ($instance['layout'] == 'masonry') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'masonry").justifiedGallery({
							lastRow: "justify",
							rowHeight: 75,
						});';
					}

					if ($instance['layout'] == 'slider') {
						echo '$("#" + response.widget_id + " .' . self::$prefix_dash . 'slider").flexslider({
							animation: "slide",';


						if (isset($instance[ 'show_nav' ]) && $instance[ 'show_nav' ]) {
							echo 'directionNav: true,';
						} else {
							echo 'directionNav: false,';
						}

						echo '});';
					}

				echo ';});
			</script>';
		}

		if ($instance['style_override']) {

			?><style>

				/* Background Color */

/*
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tile-item {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.8);
				}
*/

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.9);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>inline-card-wrap {
					background-color: rgb(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>stacked-card-wrap {
					background-color: rgb(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>colored-tabs-item {
					background-color: rgb(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-item {
					background-color: rgb(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-image .<?php echo self::$prefix_dash; ?>title,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-image .<?php echo self::$prefix_dash; ?>meta-data {
					background-color: rgba(<?php echo $r; ?>, <?php echo $g; ?>, <?php echo $b; ?>, 0.5);
				}

				/* Text Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>title {font-weight: bold;}

				#<?php echo $this->id; ?> a,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>title,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>comment-count,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>meta-data {
					color: rgb(<?php echo $text_r; ?>, <?php echo $text_g; ?>, <?php echo $text_b; ?>);
				}

			</style><?php
		} else {
			?><style>

				/* Background Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>inline-card-wrap {
					background-color: rgba(0, 0, 0, 0.8);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>stacked-card-wrap {
					background-color: rgba(0, 0, 0, 0.8);
				}

/*
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tile-item {
					background-color: rgba(0, 0, 0, 0.8);
				}
*/

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-item {
					background-color: white;
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>colored-tabs-item {
					background-color: white;
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					background-color: rgba(0, 0, 0, 0.9);
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-image .<?php echo self::$prefix_dash; ?>title,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-image .<?php echo self::$prefix_dash; ?>meta-data {
					background-color: rgba(0, 0, 0, 0.5);
				}

				/* Text Color */

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>title {
					color: white;
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>comment-count,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-tiles .<?php echo self::$prefix_dash; ?>meta-data *  {
					color: white;
				}

				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-content a,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-content .<?php echo self::$prefix_dash; ?>title,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-content .<?php echo self::$prefix_dash; ?>comment-count,
				#<?php echo $this->id; ?> .<?php echo self::$prefix_dash; ?>image-flip-content .<?php echo self::$prefix_dash; ?>meta-data {
					color: white;
				}

			</style><?php
		}

		// Include masonry files

		if ($instance['layout'] == 'masonry') {
			wp_enqueue_script(self::$prefix_dash . 'justified_gallery_js', plugins_url('core/includes/masonry/jquery.justifiedGallery.min.js', __FILE__), array('jquery'));
			wp_enqueue_style(self::$prefix_dash . 'justified_gallery_css', plugins_url('core/includes/masonry/justifiedGallery.min.css', __FILE__));
		}

		// Include slider files

		if ($instance['layout'] == 'slider') {
			wp_enqueue_script(self::$prefix_dash . 'slider_js', plugins_url('core/includes/flex-slider/jquery.flexslider-min.js', __FILE__), array('jquery'));
			wp_enqueue_style(self::$prefix_dash . 'slider_css', plugins_url('core/includes/flex-slider/flexslider.css', __FILE__));
		}

		wp_enqueue_style(self::$prefix_dash . 'widget_css', plugins_url('core/css/nnr-related-posts.css', __FILE__));
		wp_enqueue_style( 'dashicons' );

		echo $args['after_widget'];
	}

	/**
	 * Back-end widget form.
	 *
	 * @see WP_Widget::form()
	 *
	 * @param array $instance Previously saved values from database.
	 */
	public function form( $instance ) {

		// Title

		if ( isset( $instance[ 'title' ] ) ) {
			$title = $instance[ 'title' ];
		} else {
			$title = __('Related Posts', self::$text_domain );
		}

		// Layout

		if ( isset( $instance[ 'layout' ] ) ) {
			$layout = $instance[ 'layout' ];
		} else {
			$layout = __('inline_card', self::$text_domain );
		}

		// Fancy Colors

		if ( isset( $instance[ 'fancy_colors' ] ) ) {
			$fancy_colors = $instance[ 'fancy_colors' ];
		} else {
			$fancy_colors = __('#D14841,#F37934,#FAC51C,#61BD6D,#2C82C9,#553982', self::$text_domain );
		}

		// Color

		if ( isset( $instance[ 'color' ] ) ) {
			$color = $instance[ 'color' ];
		} else {
			$color = __('#FFFFFF', self::$text_domain );
		}

		// Text Color

		if ( isset( $instance[ 'text_color' ] ) ) {
			$text_color = $instance[ 'text_color' ];
		} else {
			$text_color = __('#000000', self::$text_domain );
		}

		// Num of Posts

		if ( isset( $instance[ 'num_of_posts' ] ) ) {
			$num_of_posts = $instance[ 'num_of_posts' ];
		} else {
			$num_of_posts = __('6', self::$text_domain );
		}

		// Num of Columns

		if ( isset( $instance[ 'num_of_columns' ] ) ) {
			$num_of_columns = $instance[ 'num_of_columns' ];
		} else {
			$num_of_columns = __('2', self::$text_domain );
		}

		// Grid Size

		if ( isset( $instance[ 'grid_size' ] ) ) {
			$grid_size = $instance[ 'grid_size' ];
		} else {
			$grid_size = 'medium';
		}

		// Show Category

		if ( isset( $instance[ 'show_category' ] ) ) {
			$show_category = $instance[ 'show_category' ];
		} else {
			$show_category = false;
		}

		// Show Author

		if ( isset( $instance[ 'show_author' ] ) ) {
			$show_author = $instance[ 'show_author' ];
		} else {
			$show_author = false;
		}

		// Show Post Date

		if ( isset( $instance[ 'show_post_date' ] ) ) {
			$show_post_date = $instance[ 'show_post_date' ];
		} else {
			$show_post_date = false;
		}

		// Show Comments

		if ( isset( $instance[ 'show_comments' ] ) ) {
			$show_comments = $instance[ 'show_comments' ];
		} else {
			$show_comments = false;
		}

		// Async

		if ( isset( $instance[ 'async' ] ) ) {
			$async = $instance[ 'async' ];
		} else {
			$async = true;
		}

		// Style Override

		if ( isset( $instance[ 'style_override' ] ) ) {
			$style_override = $instance[ 'style_override' ];
		} else {
			$style_override = true;
		}

		// Show Nav

		if ( isset( $instance[ 'show_nav' ] ) ) {
			$show_nav = $instance[ 'show_nav' ];
		} else {
			$show_nav = true;
		}

		// Check if there are custom post types

		$custom_post_types = get_post_types(array('public'   => true,'_builtin' => false));

		if (count($custom_post_types) == 0) {
			$custom_post_types = false;
		} else {
			$custom_post_types = true;
		}

		$id_prefix = $this->id . self::$prefix_dash;

		$taxonomies = get_taxonomies(array('public' => true));
		unset($taxonomies['post_format']);

		// View

		include(NNROBOTS_RELATED_POSTS_PLUGIN_DIR . '/admin/views/widget-settings.php');

		// Styles

		wp_enqueue_style('wp-color-picker');
		wp_enqueue_style(self::$prefix_dash . 'selectize_css', plugins_url('admin/css/selectize.css', __FILE__));

		// Scripts

		wp_enqueue_script(self::$prefix_dash . 'widget_settings_js', plugins_url('admin/js/widget-settings.js', __FILE__), array( 'wp-color-picker' ), false, true );
		wp_enqueue_script(self::$prefix_dash . 'selectize_js', plugins_url('admin/js/selectize.min.js', __FILE__), array('jquery', 'jquery-ui-core', 'jquery-ui-sortable'));
		wp_localize_script(self::$prefix_dash . 'widget_settings_js', 'nnr_rlp_widget_settings', array(
			'prefix'		=> self::$prefix_dash,
			'id_prefix'		=> $id_prefix
		));

	}

	/**
	 * Sanitize widget form values as they are saved.
	 *
	 * @see WP_Widget::update()
	 *
	 * @param array $new_instance Values just sent to be saved.
	 * @param array $old_instance Previously saved values from database.
	 *
	 * @return array Updated safe values to be saved.
	 */
	public function update( $new_instance, $old_instance ) {
		$instance = array();

		$instance['title']                        = ( ! empty( $new_instance['title'] ) ) ?
													sanitize_text_field( $new_instance['title'] ) : '';

		$instance['layout']                       = ( ! empty( $new_instance['layout'] ) ) ?
													sanitize_text_field( $new_instance['layout'] ) : '';

		$instance['fancy_colors']                 = ( ! empty( $new_instance['fancy_colors'] ) ) ?
													$new_instance['fancy_colors'] : '';

		$instance['color']                        = ( ! empty( $new_instance['color'] ) ) ?
													sanitize_text_field( $new_instance['color'] ) : '';

		$instance['text_color']                   = ( ! empty( $new_instance['text_color'] ) ) ?
													sanitize_text_field( $new_instance['text_color'] ) : '';

		$instance['num_of_posts']                 = ( ! empty( $new_instance['num_of_posts'] ) ) ?
													sanitize_text_field( $new_instance['num_of_posts'] ) : '';

		$instance['num_of_columns']               = ( ! empty( $new_instance['num_of_columns'] ) ) ?
													sanitize_text_field( $new_instance['num_of_columns'] ) : '';

		$instance['numbered_list_style']          = ( ! empty( $new_instance['numbered_list_style'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_style'] ) : '';

		$instance['numbered_list_color']          = ( ! empty( $new_instance['numbered_list_color'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_color'] ) : '';

		$instance['numbered_list_circle_color']   = ( ! empty( $new_instance['numbered_list_circle_color'] ) ) ?
													sanitize_text_field( $new_instance['numbered_list_circle_color'] ) : '';

		$instance['grid_size']                    = ( ! empty( $new_instance['grid_size'] ) ) ?
													sanitize_text_field( $new_instance['grid_size'] ) : '';

		$instance['thumbnail']                    = ( ! empty( $new_instance['thumbnail'] ) ) ?
													sanitize_text_field( $new_instance['thumbnail'] ) : '';

		$instance['show_category']                = ( ! empty( $new_instance['show_category'] ) ) ?
													sanitize_text_field( $new_instance['show_category'] ) : '';

		$instance['show_author']                  = ( ! empty( $new_instance['show_author'] ) ) ?
													sanitize_text_field( $new_instance['show_author'] ) : '';

		$instance['show_post_date']               = ( ! empty( $new_instance['show_post_date'] ) ) ?
													sanitize_text_field( $new_instance['show_post_date'] ) : '';

		$instance['show_comments']                = ( ! empty( $new_instance['show_comments'] ) ) ?
													sanitize_text_field( $new_instance['show_comments'] ) : '';

		$instance['async']                        = ( ! empty( $new_instance['async'] ) ) ?
													sanitize_text_field( $new_instance['async'] ) : '';

		$instance['style_override']               = ( ! empty( $new_instance['style_override'] ) ) ?
													sanitize_text_field( $new_instance['style_override'] ) : '';

		$instance['show_nav']                     = ( ! empty( $new_instance['show_nav'] ) ) ?
													sanitize_text_field( $new_instance['show_nav'] ) : '';

		return $instance;
	}

	/**
	 * Display popular posts based on list
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @param mixed $posts
	 * @return void
	 */
	public static function display_related_posts($instance, $post, $shortcode = false) {

		$settings = get_option(NNRobots_Related_Posts::$prefix . 'settings');

		// Default values

		if ($settings === false) {
			$settings = NNRobots_Related_Posts::$default;
		}

		// Get Related Posts

		$dates                			= self::getStartEndDates($instance);
		$start_date           			= $dates[0];
		$end_date             			= $dates[1];

		$related_posts 					= self::getRelatedPosts($instance, $post, $start_date, $end_date);

		// Check if any posts were even retrieved

		if ( count($related_posts) < 1 ) {
			return null;
		}

		$classes = '';
		$id 	 = '';
		$content = '';

		$content = '';

		if ( $shortcode ) {
			$id = $instance['id'];
			$classes .= ' ' . self::$prefix_dash . 'shortcode';
		}

		if ( isset($instance[ 'meta_data' ]) && $instance[ 'meta_data' ] ) {
			$classes .= ' ' . self::$prefix_dash . 'meta_data';
		}

		if ( $shortcode ) {
			$content .= '<div id="' . $id . '" class="' . $classes . '">';
			$content .= '<h2 class="' . self::$prefix_dash . 'shortcode-title">' . $instance['title'] . '</h2>';
		}

		// List
		if (isset($instance['layout']) && $instance['layout'] == 'list') { $content .= '<ul class="' . self::$prefix_dash . 'list">'; }

		// Numbered List
		if (isset($instance['layout']) && $instance['layout'] == 'numbered_list') { $content .= '<ol class="' . self::$prefix_dash . 'numbered-list ' . self::$prefix_dash . 'numbered-list-' . $instance['numbered_list_style'] . '">'; }

		// Image Tiles
		if (isset($instance['layout']) && $instance['layout'] == 'image_tile') { $content .= '<div class="' . self::$prefix_dash . 'image-tiles">';}

		// Inline Card
		if (isset($instance['layout']) && $instance['layout'] == 'inline_card') { $content .= '<div class="' . self::$prefix_dash . 'inline-card">'; }

		// Stacked Card
		if (isset($instance['layout']) && $instance['layout'] == 'stacked_card') { $content .= '<div class="' . self::$prefix_dash . 'stacked-card">'; }

		// Thumbnail
		if (isset($instance['layout']) && $instance['layout'] == 'thumbnail') { $content .= '<div class="' . self::$prefix_dash . 'thumbnail">'; }

		// Masonry
		if (isset($instance['layout']) && $instance['layout'] == 'masonry') { $content .= '<div class="' . self::$prefix_dash . 'masonry">';}

		// Grid
		if (isset($instance['layout']) && $instance['layout'] == 'grid') { $content .= '<div class="' . self::$prefix_dash . 'grid ' . self::$prefix_dash . 'grid-' . $instance['grid_size'] . '">';}

		// Slider
		if (isset($instance['layout']) && $instance['layout'] == 'slider') { $content .= '<div class="' . self::$prefix_dash . 'slider flexslider"><ul class="slides">';}

		// Fancy Colors
		if (isset($instance['layout']) && $instance['layout'] == 'fancy_colors') {$content .= '<div class="' . self::$prefix_dash . 'fancy-colors">';}

		// Colored Tabs
		if (isset($instance['layout']) && $instance['layout'] == 'colored_tabs') {$content .= '<div class="' . self::$prefix_dash . 'colored-tabs">';}

		// Image Flip
		if (isset($instance['layout']) && $instance['layout'] == 'image_flip') {$content .= '<div class="' . self::$prefix_dash . 'image-flip">';}

		/**
		 * The loop used to display all related posts
		 */

		$num_of_posts = 0;

		foreach ($related_posts as $related_post) {

			// Break if number of posts is met

			if ($instance['num_of_posts'] == $num_of_posts) {
				break;
			}

			// Post Title

			$post_title = '<a href="' . get_permalink($related_post->ID) . '" class="' . self::$prefix_dash . 'title" title="' . $related_post->post_title . '">' . $related_post->post_title . '</a>';

			// Post meta data

			$post_meta_data = '<div class="' . self::$prefix_dash . 'meta-data">';

			if ($instance['layout'] != 'masonry' && $instance['layout'] != 'grid') {

				if ($shortcode && $instance['id'] == 'nnrobots_related_posts_0') {

					$instance['show_comments']     = $settings['show_comments'];
					$instance['show_category']     = $settings['show_category'];
					$instance['show_author']       = $settings['show_author'];
					$instance['show_post_date']    = $settings['show_post_date'];

				}

				// Comments Count

				$comment_count = '';

				if ( isset($instance['show_comments']) && $instance['show_comments'] ) {
					$comment_count = '<a class="' . self::$prefix_dash . 'comment-count" href="' . get_comments_link($related_post->ID) . '"><span class="dashicons dashicons-admin-comments"></span>' . get_comments_number($related_post->ID) . '</a>';
				}

				// Category

				$category = '';

				if ( isset($instance['show_category']) && $instance['show_category'] ) {
					$category 		= get_the_category( $related_post->ID );

					if ( isset($category) && is_array($category) && count($category) > 0 ) {
						$category_link 	= get_category_link( $category[0]->term_id );
						$category 		= '<a href="' . esc_url( $category_link ) . '" class="' . self::$prefix_dash . 'category" title="' . $category[0]->cat_name . '"><span class="dashicons dashicons-category"></span>' . $category[0]->cat_name . '</a>';
					} else {
						$category = '';
					}
				}

				// Author

				$author = '';

				if ( isset($instance['show_author']) && $instance['show_author'] ) {

					$author_data 	= get_userdata( $related_post->post_author );
					$author 		= '<a href="' . get_author_posts_url( $related_post->post_author ) . '" class="' . self::$prefix_dash . 'author" title="' . $author_data->user_nicename . '"><span class="dashicons dashicons-admin-users"></span>' . $author_data->user_nicename . '</a>';
				}

				// Date

				$date = '';

				if ( isset($instance['show_post_date']) && $instance['show_post_date'] ) {

					$display_date 	= date('M j, Y', strtotime($related_post->post_date));
					$archive_year  	= date('Y', strtotime($related_post->post_date));
					$archive_month 	= date('m', strtotime($related_post->post_date));
					$archive_day   	= date('d', strtotime($related_post->post_date));
					$date 			= '<a href="' .  get_day_link($archive_year, $archive_month, $archive_day) . '" class="' . self::$prefix_dash . 'date" title="' . $display_date . '"><span class="dashicons dashicons-calendar"></span>' . $display_date . '</a>';
				}

				$post_meta_data .= $category . $author . $date . $comment_count;
			}

			$post_meta_data .= '</div>';

			switch ( $instance['layout'] ) {

				case 'list':

					$content .= '<li>' . $post_title . $post_meta_data . '</li>';

				break;

				case 'inline_card':

					$image = wp_get_attachment_image_src( get_post_thumbnail_id($related_post->ID) , 'large');

					if ( $image ) {

						if ( is_array($image) ) {
							$image = $image[0];
						}

					} else {

						$image = wp_get_attachment_image_src( $settings["default_thumbnail"], 'large');

						if ( $image ) {
							$image = $image[0];
						} else {
							$image = plugins_url('img/related-default-thumbnail.jpg', __FILE__);
						}

					}

					$content .= '<div class="' . self::$prefix_dash . 'inline-card-item ' . self::$prefix_dash . 'inline-card-item-2">
						<div class="' . self::$prefix_dash . 'inline-card-wrap">
							<a class="' . self::$prefix_dash . 'inline-card-image-container" style="background-image: url(' . $image . '); " href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '"></a>
							<div class="' . self::$prefix_dash . 'inline-card-data-container">' . $post_title . $post_meta_data . '</div>
						</div>
					</div>';

				break;

				case 'stacked_card':

					$image = wp_get_attachment_image_src( get_post_thumbnail_id($related_post->ID) , 'large');

					if ( $image ) {

						if ( is_array($image) ) {
							$image = $image[0];
						}

					} else {

						$image = wp_get_attachment_image_src( $settings["default_thumbnail"], 'large');

						if ( $image ) {
							$image = $image[0];
						} else {
							$image = plugins_url('img/related-default-thumbnail.jpg', __FILE__);
						}

					}

					$content .= '<div class="' . self::$prefix_dash . 'stacked-card-item ' . self::$prefix_dash . 'stacked-card-item-' . $instance["num_of_columns"] . '">
						<div class="' . self::$prefix_dash . 'stacked-card-wrap">
							<a class="' . self::$prefix_dash . 'stacked-card-image-container" style="background-image: url(' . $image . '); " href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '"></a>
							<div class="' . self::$prefix_dash . 'stacked-card-data-container">' . $post_title . $post_meta_data . '</div>
						</div>
					</div>';

				break;

				case 'thumbnail':

					$thumbnail = get_the_post_thumbnail( $related_post->ID, 'thumbnail', array('class'	=> self::$prefix_dash . 'thumbnail-image'));
					$image = wp_get_attachment_image_src( get_post_thumbnail_id($related_post->ID) , 'thumbnail');

					if ( !isset($thumbnail) || empty($thumbnail) ) {

						$image = wp_get_attachment_image_src($settings["default_thumbnail"], 'thumbnail');

						if ( $image ) {
							$thumbnail = '<img class="' . self::$prefix_dash . 'thumbnail-image" src="' . $image . '"/>';
						} else {
							$thumbnail = '<img class="' . self::$prefix_dash . 'thumbnail-image" src="' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . '"/>';
						}

					}

					$content .= '<div class="' . self::$prefix_dash . 'thumbnail-item">
						<div class="' . self::$prefix_dash . 'thumbnail-image-container">
							<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '">' . $thumbnail . '</a>
						</div>
						<div class="' . self::$prefix_dash . 'thumbnail-data-container">' . $post_title . $post_meta_data . '</div>
					</div>';


				break;

				case 'image_tile':

					$thumbnail = get_the_post_thumbnail( $related_post->ID, 'large');
					$image = wp_get_attachment_image_src( get_post_thumbnail_id($related_post->ID) , 'large');

					if ( !isset($thumbnail) || empty($thumbnail) ) {

						$image = wp_get_attachment_image_src( $settings["default_thumbnail"] , 'large');

						if ( $image ) {
							$thumbnail = '<img class="' . self::$prefix_dash . 'thumbnail-image" src="' . $image[0] . '"/>';
						} else {
							$thumbnail = '<img class="' . self::$prefix_dash . 'thumbnail-image" src="' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . '"/>';
						}

					}

					$content .= '
					<div class="' . self::$prefix_dash . 'image-tile-item">' .
							$thumbnail .
						'<div>' .
							$post_title .
						'</div>' .
						'<div class="' . self::$prefix_dash . 'image-tile-data">' .
							$post_meta_data .
						'</div>
					</div>';

				break;

				case 'masonry':

					$image = wp_get_attachment_image_src(get_post_thumbnail_id($related_post->ID), 'large');

					if ( !isset($image) || empty($image)) {
						$image = wp_get_attachment_image_src($settings["default_thumbnail"], 'large');
					}

					if ( $image ) {
						$content .= '<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '"><img alt="' . $related_post->post_title . '" src="' . $image[0] . '" /></a>';
					} else {
						$content .= '<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '"><img alt="' . $related_post->post_title . '" src="' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . '" /></a>';
					}

				break;

				case 'grid':

					if ($shortcode) {
						$image = wp_get_attachment_image_src(get_post_thumbnail_id($related_post->ID), 'large');
					} else {
						$image = wp_get_attachment_image_src(get_post_thumbnail_id($related_post->ID));
					}

					if ( !isset($image) || empty($image)) {
						$image = wp_get_attachment_image_src($settings["default_thumbnail"], 'large');
					}

					if ($image) {
						$content .= '<div class="' . self::$prefix_dash . 'grid-item"><a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '" style="background-image: url(' . $image[0] . ');"></a></div>';
					} else {
						$content .= '<div class="' . self::$prefix_dash . 'grid-item"><a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '" title="' . $related_post->post_title . '" style="background-image: url(' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . ');"></a></div>';
					}

				break;

				case 'slider':

					$image = wp_get_attachment_image_src(get_post_thumbnail_id($related_post->ID), 'large');

					if ( !isset($image) || empty($image)) {
						$image = wp_get_attachment_image_src($settings["default_thumbnail"], 'large');
					}

					if ($image) {
						$content .= '<li><img alt="' . $related_post->post_title . '" src="' . $image[0] . '" />';
						$content .= '<div class="image-caption">
							<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '">' . $related_post->post_title . '</a>' .
							$post_meta_data .
						'</div>';
						$content .= '</li>';
					} else {
						$content .= '<li><img alt="' . $related_post->post_title . '" src="' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . '" />';
						$content .= '<div class="image-caption">
							<a class="' . self::$prefix_dash . 'title" href="' . get_permalink($related_post->ID) . '">' . $related_post->post_title . '</a>' .
							$post_meta_data .
						'</div>';
						$content .= '</li>';
					}

				break;

				case 'fancy_colors':

					$fancy_colors = explode(',', $instance['fancy_colors']);

					if ($num_of_posts >= count($fancy_colors)) {
						$color = $fancy_colors[count($fancy_colors) - 1];
					} else {
						$color = $fancy_colors[$num_of_posts];
					}

					$content .= '<div class="' . self::$prefix_dash . 'fancy-colors-content" style="background-color: ' . $color . ';">' .
						$post_title . $post_meta_data .
					'</div>';

				break;

				case 'colored_tabs':

					$fancy_colors = explode(',', $instance['fancy_colors']);

					if ($num_of_posts >= count($fancy_colors)) {
						$color = $fancy_colors[count($fancy_colors) - 1];
					} else {
						$color = $fancy_colors[$num_of_posts];
					}

					$content .= '<div class="' . self::$prefix_dash . 'colored-tabs-item">
						<div class="' . self::$prefix_dash . 'colored-tabs-color" style="background-color: ' . $color . ';"></div>
						<div class="' . self::$prefix_dash . 'colored-tabs-content">' .
						$post_title . $post_meta_data .
						'</div>
					</div>';

				break;

				case 'image_flip':

					$image = wp_get_attachment_image_src(get_post_thumbnail_id($related_post->ID), 'large');
					$image_html = '';

					if ( !isset($image) || empty($image) ) {

						$image = wp_get_attachment_image_src( $settings["default_thumbnail"] , 'large');

					}

					if ($image) {
						$content .= '<div class="' . self::$prefix_dash . 'image-flip-item-wrap">
							<div class="' . self::$prefix_dash . 'image-flip-item">
								<div class="' . self::$prefix_dash . 'image-flip-content">' .
									'<a class="' . self::$prefix_dash . 'title">' . $related_post->post_title . '</a>' . $post_meta_data .
									'</div>
									<div class="' . self::$prefix_dash . 'image-flip-image" style="background-image: url(' . $image[0] . ');">' . $post_title . $post_meta_data . '</div>
							</div>
						</div>';
					} else {
						$content .= '<div class="' . self::$prefix_dash . 'image-flip-item-wrap">
							<div class="' . self::$prefix_dash . 'image-flip-item">
								<div class="' . self::$prefix_dash . 'image-flip-content">' .
									'<a class="' . self::$prefix_dash . 'title">' . $related_post->post_title . '</a>' . $post_meta_data .
									'</div>
								<div class="' . self::$prefix_dash . 'image-flip-image" style="background-image: url(' . plugins_url('img/related-default-thumbnail.jpg', __FILE__) . ');">' . $post_title . $post_meta_data . '</div>
							</div>
						</div>';
					}

				break;
			}

			$num_of_posts++;
		}

		// List
		if (isset($instance['layout']) && $instance['layout'] == 'list') { $content .= '</ul>'; }

		// Numbered List
		if (isset($instance['layout']) && $instance['layout'] == 'numbered_list') { $content .= '</ol>'; }

		// Image Tiles
		if (isset($instance['layout']) && $instance['layout'] == 'image_tile') { $content .= '</div>';}

		// Inline Card
		if (isset($instance['layout']) && $instance['layout'] == 'inline_card') { $content .= '</div>'; }

		// Stacked Card
		if (isset($instance['layout']) && $instance['layout'] == 'stacked_card') { $content .= '</div>'; }

		// Thumbnail
		if (isset($instance['layout']) && $instance['layout'] == 'thumbnail') { $content .= '</div>'; }

		// Masonry
		if (isset($instance['layout']) && $instance['layout'] == 'masonry') { $content .= '</div>';}

		// Grid
		if (isset($instance['layout']) && $instance['layout'] == 'grid') { $content .= '</div>';}

		// Slider
		if (isset($instance['layout']) && $instance['layout'] == 'slider') { $content .= '</ul></div>';}

		// Fancy Colors
		if (isset($instance['layout']) && $instance['layout'] == 'fancy_colors') {$content .= '</div>';}

		// Colored Tabs
		if (isset($instance['layout']) && $instance['layout'] == 'colored_tabs') {$content .= '</div>';}

		// Image Flip
		if (isset($instance['layout']) && $instance['layout'] == 'image_flip') {$content .= '</div>';}

		if ( $shortcode ) {
			$content .= '</div>';
		}

		return $content;
	}

	/**
	 * Gets all posts based on user widget settings
	 *
	 * @access public
	 * @static
	 * @param mixed $instance
	 * @return array $posts
	 */
	public static function getRelatedPosts($instance, $post, $start_date, $end_date) {

		// Convert post object or string to array

		if ( is_object($post) ) {
			$post = (array) $post;
		}

		if ( !is_array($post) && is_string($post) ) {
			$post = (array) json_decode($post);
		}

		if ( !isset($instance['id']) ) {
			$instance['id'] = 'nnr_rlp_id_';
		}

		// Check if we have cached data

		if ( false !== ( $posts = get_transient( 'nnr_rlp_cached_posts_' . $post['ID'] . '_' . $instance['id'] ) ) ) {
			return $posts;
		}

		$settings = get_option(NNRobots_Related_Posts::$prefix . 'settings');

		// Default values

		if ($settings === false) {
			$settings = NNRobots_Related_Posts::$default;
		}

		// Check if user set manual posts

		$related_posts = get_post_meta( $post['ID'], NNRobots_Related_Posts::$prefix . 'meta_data' , true );

		if ( isset($related_posts) && $related_posts != '') {

			$posts = get_posts(array(
				'post__in' 	=> explode(',', $related_posts),
				'orderby'	=> 'post__in'
			));

			// If no posts were retreived then just get the recent posts

			if (isset($posts) && is_array($posts) && count($posts) < 1) {
				$posts = get_posts(array(
					'posts_per_page' 	=> $instance['num_of_posts'],
					'post_status' 		=> 'publish',
					'post__not_in'		=> array($post['ID']),
				));
			}

			return $posts;
		}

		// Determine what posts to get

		$post_args = array(
			'nnr_post_id'			 => $post['ID'],
			'posts_per_page'         => $instance['num_of_posts'],
			'offset'                 => 0,
			'post_status'            => 'publish',
			'post_type'              => $post['post_type'],
			'order'                  => $settings['order'],
			'order_by'               => $settings['order_by'],
			'numberposts'            => $instance['num_of_posts'],
			'post__not_in'			 => array($post['ID']),
			'offset'                 => 0,
			'include'                => array(),
			'exclude'                => $post['ID'],
			'meta_key'               => '',
			'meta_value'             => '',
			'suppress_filters'       => true,
			'ignore_sticky_posts'    => true,
			'no_found_rows'          => true,
		);

		// Date Range

		$post_args['date_query'] = array(
			array(
				'after'     => $start_date,
				'before'    => $end_date,
				'inclusive' => true,
			),
		);

		// Exclude posts

		if ( $settings['display_in'] != 'any' && $settings['query'][$post['post_type']]['exclude'] != '' ) {
			$post_args['post__not_in'] = array_merge($post_args['post__not_in'], explode(',', $settings['query'][$post['post_type']]['exclude']));
		} else {
			$post_types = get_post_types(array('public' => true));
			unset($post_types['attachment']);

			foreach ($post_types as $post_type) {
				if ($settings['query'][$post_type]['exclude'] != '') {
					$post_args['post__not_in'] = array_merge($post_args['post__not_in'], explode(',', $settings['query'][$post_type]['exclude']));
				}
			}
		}

		// Exclude Posts by Taxonomy

		if ( isset($settings['query']['taxonomy']) && is_array($settings['query']['taxonomy']) ) {

			foreach ($settings['query']['taxonomy'] as $taxonomy => $value) {

				if ($value['exclude'] != '') {
					$post_args['tax_query'][] = array(
						'taxonomy'  => $taxonomy,
						'field'     => 'term_id',
						'terms'     => explode(',', $value['exclude']),
						'operator'  => 'NOT IN'
					);
				}
			}

		}

		// Query posts based on taxonomy

		if ( $settings['related_by'] != 'contextual' ) {

			// Get Related Posts by taxonomy

			$post_args['tax_query'] = array();
			$post_args['tax_query']['relation'] = 'AND';

			$terms = wp_get_post_terms($post['ID'], $settings['related_by']);

			if ($terms) {

				$term_ids = array();

				foreach($terms as $term)  {
					$term_ids[] = $term->term_id;
				}

				$post_args['tax_query'][] = array(
					'taxonomy'  => $settings['related_by'],
					'field'     => 'term_id',
					'terms'     => $term_ids,
					'operator'  => 'IN'
				);
			}
		}

		// Query posts based on Post Title

		else {

			$post_args['s'] = wp_trim_words( $post['post_title'] , 9 );

		}

		// Add our custom filter

		add_filter( 'posts_search', array('NNRobots_Related_Posts_Widget', 'filter_search'), 1, 2);

		$posts_query = new WP_Query;
		$posts = $posts_query->query($post_args);

		// Remove our custom filter

		remove_filter('posts_search', array('NNRobots_Related_Posts_Widget', 'filter_search'));

		// If no posts were retreived then just get the recent posts

		if (isset($settings['add_recent']) &&
			$settings['add_recent'] &&
			isset($posts) &&
			is_array($posts) &&
			count($posts) < $instance['num_of_posts']) {

			$post_ids = array();
			foreach ($posts as $val) {
				$post_ids[] = $val->ID;
			}

			$recent_posts = get_posts(array(
				'posts_per_page' 	=> $instance['num_of_posts'] - count($posts),
				'post_status' 		=> 'publish',
				'post__not_in'		=> array_merge(array($post_args['post__not_in']), $post_ids),
			));

			$posts = array_merge($posts, $recent_posts);
		}

		set_transient( 'nnr_rlp_cached_posts_' . $post['ID'] . '_' . $instance['id'], $posts, 24 * HOUR_IN_SECONDS );

		return $posts;
	}

	/**
	 * Filters the serach parameter for a WP Query and uses ORs to concatinate
	 * the search args not ANDs.  This makes for a more true search effect
	 *
	 * @access public
	 * @static
	 * @param mixed $term
	 * @return void
	 */
	static function filter_search($serach, $object) {

		// Make sure we have the current post id

		if ( !isset($object->query['nnr_post_id']) ) {
			return $serach;
		}

		global $wpdb;

		$post = get_post($object->query['nnr_post_id']);

		if ( isset( $post ) ) {

			// Get the excerpt

			$excerpt = $post->post_excerpt;

			if ( $excerpt == '' ) {
				$excerpt = $post->post_content;
			}

			// escpae the content

			$content = preg_replace('~[^a-zÀ-ÖØ-öÿŸ\d]++~ui', ' ', strip_tags( strip_shortcodes( $post->post_title . ' ' . $excerpt ) ) );

			return $wpdb->prepare(" AND MATCH ({$wpdb->prefix}posts.post_title) AGAINST (%s IN BOOLEAN MODE) ", wp_trim_words( $content , 5000));
		}

		return $serach;
	}

	/**
	 * Get start and end dates
	 *
	 * @access public
	 * @param mixed $instance
	 * @return array($start_date, $end_date)
	 */
	public static function getStartEndDates($instance) {

		// Determine Start and End Dates

		$end_date = date("Y-m-d");

		if (isset($instance[ 'time_range' ]) && $instance[ 'time_range' ] == '1_day') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 1, date("Y")));
		} else if (isset($instance[ 'time_range' ]) && $instance[ 'time_range' ] == '7_days') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 7, date("Y")));
		} else if (isset($instance[ 'time_range' ]) && $instance[ 'time_range' ] == '30_days') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j") - 30, date("Y")));
		} else if (isset($instance[ 'time_range' ]) && $instance[ 'time_range' ] == '1_year') {
			$start_date = date("Y-m-d", mktime(0, 0, 0, date("n"), date("j"), date("Y") - 1));
		} else {
			$start_date = '';
		}

		return array($start_date, $end_date);
	}

} // class NNRobots_Related_Posts_Widget